package it.jrc.entitymatcher;

// the logger
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Entity
{
	private static Logger logger = LoggerFactory.getLogger("it.jrc.entitymatcher.Entity");

	public int id;
	public char type;
	public String lan;
	public String alias;
	public int languageCode;
	public int charIndex;

	public Entity()
	{
	}


	public static char[] charID(int id)
	{
		char[] cid = new char[2];
		cid[0] = (char)((id & 0xffff0000) >> 16);
		cid[1] = (char)(id & 0x0000ffff);
		return cid;
	}

	private int setID(char[] charID, int index)
	{
		int id = ((int)charID[index] << 16) | charID[index+1];
		return id;
	}

	public Entity(char[]data, int index, String word, int cIndex)
	{
		alias = word;
		id = setID(data, index);
	  index = index+2;
	  type = data[index++];
	  languageCode = data[index++];
	  charIndex = cIndex;
	}


	/**
	 * 	Parse an entity from a line
	 *	the fomat of the line should be (whitespace separated)
	 *	id type language alias (note that in this implementation any - signs are ignored... )
	 *	id: the numeric id of the entity (Integer)
	 *	type: one character, will be set to lowercase, normally p (person) or o (other)
	 *	language: 'u' for all languages or 2 character ISO language code (very occasionally 3 e.g. 'pap') will be set to lowercase
	 *	alias: a string representing the entity (possibly restricted by language code)
	 */
	public void init(String line)
	{
		try
		{
			int i = 0;
			// deal with possible '-' sign
			if (line.charAt(0) == '-') i++;
			while (Character.isDigit(line.charAt(i++)));
			id = Integer.parseInt(line.substring(0,i-1));
			while (i < line.length() && Character.isWhitespace(line.charAt(i++)));
			type = Character.toLowerCase(line.charAt(i-1));
			while (i < line.length() && Character.isWhitespace(line.charAt(i++)));
			int start = i-1;
			while (i < line.length() && Character.isLetter(line.charAt(i++))) ;
			lan = line.substring(start,i-1).toLowerCase();
			while (i < line.length() && Character.isWhitespace(line.charAt(i++)));
			if (i < line.length()) alias = line.substring(i-1).trim();
		}
		catch(Exception e){logger.error("could not parse entity from "+line, e);}
	}


	/**
	 * 	returns the entity metadata encoded in a char[4] array
	 *	2 characters for the ineteger ID (32 bits)
	 *	1 character for the type
	 *	1 character for the INDEX in the code2lan array
	 */
	public char[] charCode()
	{
		char[] code = new char[4];
		code[0] = charID(id)[0];
		code[1] = charID(id)[1];
		code[2] = type;
		code[3] = (char)languageCode;
		return code;
	}

	public void appendTo(StringBuilder sb)
	{
		sb.append(charID(id));
		sb.append(type);
		sb.append((char)languageCode);
	}

}
