package it.jrc.entitymatcher;

import java.util.Hashtable;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Set;
import it.jrc.matcher.MatchResult;


/*
 *	Match Result holds the match results for the text passed through MatchWords.
 *	It stores these results on a per entity basis
 */
public class EntityMatchResult implements MatchResult
{
	private static final boolean DEBUG = false;

	private HashMap<Integer, ArrayList<String>> entities;
	private HashMap<Integer, Character> types;
  private HashMap<Integer, ArrayList<Integer>> positions;
  private char textLanguageCode;
	private int nWordCount;

  public EntityMatchResult(int languageCode)
  {
    entities = new HashMap<Integer, ArrayList<String>>();
    positions = new HashMap<Integer, ArrayList<Integer>>();
    types = new HashMap<Integer, Character>();
		textLanguageCode = (char)languageCode;
    nWordCount = 0;
  }

	public void reset()
	{
		entities.clear();
		types.clear();
    positions.clear();
		nWordCount = 0;
	}

	// add this alias to the entity
  private void addAlias(Entity e)
  {
  	// but only if the entity id != 0!!
  	if (e.id != 0)
  	{
	  	ArrayList<String> aliasList = entities.get(e.id);
			ArrayList<Integer> oPos = positions.get(e.id);
			if (aliasList == null)
			{
				aliasList = new ArrayList<String>();
				entities.put(e.id, aliasList);
				types.put(e.id, e.type);
	      oPos = new ArrayList<Integer>();
	      positions.put(e.id, oPos);
			}
	    oPos.add(e.charIndex);
			aliasList.add(e.alias);
  	}
  }

	/**
	 *	addMatch is called when a string in the text matches a pattern (i.e.&nbsp; an alias)
	 *	this alias can itself represent more than one entity!!
	 *
	 *	@param data is the data array that contains ALL aliases and associated information structures
	 *	@param index is a pointer in this data array, it will point to the last char of the pattern
	 *	@param word is the actual string in the text that matches this pattern
	 *	@param wordIndex is the index of the word in the text
	 *	@param charIdex is the character index of the word in the text
	 *
	 */
  public void addMatch(char[] data, int index, String word, int wordIndex, int charIndex)
  {

  	int infoIndex = index+2;
 		int patternLen = (int)data[infoIndex++];
    String pattern = String.valueOf(data, index-patternLen+1, patternLen);
    char count = data[infoIndex++];
		// the simple case. We have one associated entity where we either don't care about the language
		// or the language of the text corresponds to this definition
    if (count == 1)
    {
			Entity e = new Entity(data, infoIndex, word, charIndex);
  		if (e.languageCode == 0 || e.languageCode == textLanguageCode)
    	{
				if (DEBUG) System.out.println("FOUND "+word+" id = "+ e.id+" count:"+(int)count+" type = "+e.type+" language code: "+(int)e.languageCode+" char="+charIndex);
				addAlias(e);
    	}
    }
    // We have multiple entities defined against this alias (either different id or different language)
		else
		{
			Entity languageSpecificEntity = null;
			Entity generalEntity = null;
			for (int i=0; i<count; i++)
			{
				Entity e = new Entity(data, infoIndex, word, charIndex);
				infoIndex = infoIndex+4;

				if (DEBUG) System.out.println("BINGO "+word+" id = "+ e.id+" count:"+(int)count+" type = "+e.type+" language code: "+(int)e.languageCode+" char="+charIndex);

		  	if (e.languageCode == textLanguageCode)
		  		languageSpecificEntity = e;
		  	else if (e.languageCode == 0)
		  		generalEntity = e;
			}

			// now add this alias to the appropriate entity list (if any)
			// if a general entity exists (i.e. the language is set to "u") and this entity maps to 0,
			// totally ignore this entity
			if (languageSpecificEntity != null)
			{
				if (generalEntity == null)
					addAlias(languageSpecificEntity); 		// no reason not to add it
				else if (generalEntity.id != 0)
					addAlias(languageSpecificEntity);			// idem
			}
			else if (generalEntity != null && generalEntity.id != 0)
				addAlias(generalEntity);

		}
	}

	public void setWordCount(int n)										{nWordCount = n;}

	public int getWordCount()													{return nWordCount;}
	public Set<Integer> getEntities()									{return entities.keySet();}
	public Character getType(int id)									{return types.get(id);}
	public ArrayList<String> getAliases(int id)				{return entities.get(id);}
	public ArrayList<Integer> getCharIndexes(int id) 	{return positions.get(id);}


}